#include "kant.h" 
#include "rel_lattice.h" 
#include "anf_rel_sort.h" 

extern t_real fincke_func ();


t_logical 
rel_order_lat_norm_equation_mod99 WITH_11_ARGS (
	order,         		ord,
        order, 	       		sub_ord,
        order, 	       		rel_ord,
	vector,			rel_con,
        vector,        		K,
        vector*,       		G,
        anf_rel_sort,  		sort,
        dyn_arr_handle *,       sol_list, 
	t_logical,		find_all,
	anf_elt,		norm_elt,
	t_logical,		abs_value
)
/*******************************************************************************
 
Description:
 
	This function search a solution of a relative norm equation.

 
Calling sequence:
 
 	ok= rel_order_lat_norm_equation(ord, sub_ord, rel_ord, rel_con, K, G, sort, 
	                               &sol_list, find_all, norm_elt, abs_value);


        order  	          ord      = t_handle of an order 
        order  	          sub_ord  = t_handle of an sub order of ord
        order  	          rel_ord  = t_handle of an the order from the relative basis over Z
        vector		  rel_con  = t_handle of the conjugated relative basis
        vector	 	  K        = the values of the relative norm equations
				     (t_reals)
        vector	 	  G        = the values of the relative gammas (t_reals)
        anf_rel_sort	  sort     = the sorting of the conjugetes over sub_ord
        dyn_arr_handle 	  sol_list = one solution if one found
			             (vector of alg. numbers in the sub order)
	integer_small     find_all = 0, find one solution
				     1, find all solutions
	t_logical	  log	   = 1, if there is a solution
                                     0, if there is no solution
	anf_elt           norm_elt = the norm in the subfield
	t_logical	  abs_value= 1, if solution with exact the rel. norm of norm_elt
                                     0, if solution with the absolute value of the
                                           rel. norm of norm_elt

 
History:
 
	92-06-23 AJ    written
 
*******************************************************************************/
{ 

       	block_declarations;         

        t_handle        R, Z, C;

        vector	      all_ceils;    /* vector of the relative anf_ceilings */
	vector	      lambdas;      /* vector of the t_real lambdas */ 
	vector	      ubounds;      /* vector of the upper bounds of r_i^(j) (integer_samll) */ 
				    /* each entry of ubounds is a vector with the bounds     */
				    /* ober the conjugated field                             */
	vector	      lbounds;      /* vector of the lower bounds of r_i^(j) (integer_samll) */ 
				    /* each entry of ubounds is a vector with the bounds     */
				    /* ober the conjugated field                             */

                     
        rel_lattice  		rel_lat;
        rel_lat_enum_env	rel_env;
        rel_lat_elt  		rel_lat_vec;
        vector	      		norms;         /* vector of the t_real relative norms */ 
        vector	      		t2_bounds;     /* vector of the t_real t2 bounds      */ 

        dyn_arr_handle          sol_list_abs; 
    
        t_real        start, step, temp1, temp2, temp3, temp4, param1, param2, t2_bnd;
	t_real	      real_m, loceps, zero, two, sq2;
	t_comp	      ctemp1, ctemp2, ctemp3, ctemp4, ctemp5, norm_elt_c;
	anf_elt	      rel_elt, elt_h1, elt_h2, elt_h3, alpha;
        integer_small n, m, nm, i, j, l, k, sub_r1, sub_r2, sub_r1r2, modi, sj, tj, ind, r1,r2, ih1;
	integer_big   abs_norm, abs_den, index;
        t_logical     ok, test, sol_found, erg, unit_tst;                  

        matrix        rel_abs_basis, rel_abs_basis_weight; 
                      /* all (and weighted) conjugates of the relative basis  */

        matrix        lll_trans, lll_trans_comp, lll_trans_int, lll_result, mat_h1 ; 
                      /* the LLL-Transf. of the weighted conjugates of the relative basis  */
                                   
                 
        integer_small ceil_cnt, lat_vec_cnt, right_norm_cnt;

	lattice		lat;
	lat_enum_env	env;
        lat_elt         lat_vec;


                                                     

/*** Initialisation  **********************************************************/
                                            
	ceil_cnt= 0;
	lat_vec_cnt= 0;
        right_norm_cnt= 0;

        Z= m_z_str_incref(structure_z);
	R= order_reals (ord);
	C = comp_create(real_dec_prec(R));
  	nm= order_abs_degree (ord);
  	r1= order_r1(ord);
  	r2= order_r2(ord);
  	n= order_abs_degree (sub_ord);
  	sub_r1= order_r1(sub_ord);
  	sub_r2= order_r2(sub_ord);
	sub_r1r2= sub_r1 + sub_r2;
  	m= nm/n;  
	real_m= conv_int_to_real(R, m);
	zero= conv_int_to_real(R, 0);
	two=  conv_int_to_real(R, 2);
        sq2= real_sqrt(R, two);
        loceps= real_make(R, 10, -real_dec_prec(R)+2 );  
   
	norm_elt_c= anf_elt_ith_con(sub_ord, norm_elt,1);

	t2_bounds= vec_new(sub_r1r2);
	all_ceils= vec_new(sub_r1r2);

	*sol_list= dyn_arr_alloc(0);
        dyn_arr_curr_length(*sol_list)= 0;    

	if (anf_print_level >=3 )
	 {
          printf("\n\n\nOutput of alsolute basis:\n\n");
	  mat_real_write_aj(R, order_basis_real(rel_ord), 15);
	  printf("\n\n");     
	 }
 



/*** Determination of the zeroes lambda  **************************************/
           
        if (*G== MEM_NH)
         {
          rel_order_fincke_find_gamma_opt(sub_ord, K, G, &lambdas, sort);           

          printf("\nOptimale gamma's and lambdas: \n\n");
          mat_real_write_aj(R, *G, 10);          
          printf("\n \n");
          mat_real_write_aj(R, lambdas, 10);          
	  printf("\n \n");
         }
        else
          {
	   /* Start values for the regula falsi */

   	   lambdas= vec_new(sub_r1r2);

	   start  = conv_double_to_real (R, 1.1);  

	   /* Length of the steps to find the zero for the regula falsi */

   	   step   = conv_double_to_real (R,2.0);

	   /* The regula falsi */   


  	   for (j=1; j<=sub_r1r2; j++)
	    {                       
	     param1= real_divide(R, vec_entry(*G,j), vec_entry(K,j));
	     param2= conv_double_to_real (R, (double) (2.0/(1.0*m)) );

	     if (anf_print_level>=6)
	      {
	       printf("\nVALUES: \n");
               printf ("start : "); real_write (R,start,10); puts ("");
               printf ("step  : "); real_write (R,step,10);  puts ("");
               printf ("p1    : "); real_write (R,param1,10);    puts ("");
               printf ("p2    : "); real_write (R,param2,10);    puts ("");
	      } 
	
             ok = regula_falsi_zero (R,start,step,&fincke_func,
	   	                     param1,param2,&vec_entry(lambdas,j) ); 

	     real_delete(&param1);
	     real_delete(&param2);

	    }

           real_delete(&start);
           real_delete(&step);
          }


/*** Determination of the bound for r  *****************************************/

               
        rel_order_fincke_rbounds_calc(rel_ord, sub_ord, K, sort, lambdas,
                                      &ubounds, &lbounds);
    
               
/*****  t2_bounds  **************************************/
                               
        t2_bnd= real_incref(zero);
	for (j=1; j<=sub_r1r2; j++)   
	 {
	  temp2= real_add(R, vec_entry(K,j), vec_entry(*G,j));
	  temp3= real_root(R, temp2, m);
	  temp4= real_mult(R, temp3, temp3);
	  vec_entry(t2_bounds,j)= real_mult(R, temp4, real_m);
	  real_delete(&temp2);
	  real_delete(&temp3);
	  real_delete(&temp4);
  
          temp1= t2_bnd;
          t2_bnd= real_add(R, temp1, vec_entry(t2_bounds,j));
	  real_delete(&temp1);
          if (j>sub_r1)
           {
            temp1= t2_bnd;
            t2_bnd= real_add(R, temp1, vec_entry(t2_bounds,j));
  	    real_delete(&temp1);
           }
	 }                                                    


/*****  output     **************************************/

        if (anf_print_level >=0)
         {      
          if (!order_disc_known(rel_ord)) 
            order_disc_calc(rel_ord);
          if (!order_disc_known(ord)) 
            order_disc_calc(ord);
          if (!order_disc_known(sub_ord))
            order_disc_calc(sub_ord);


          printf ("(nm = %d, s  = %d)\n",nm,order_r1(rel_ord));  
          printf("Order discriminant of R = ");
	  integer_write(order_disc(rel_ord));
	  printf("\n");
          printf("Order discriminant of oK = ");
	  integer_write(order_disc(ord));
	  printf("\n");     
          index= integer_div(order_disc(rel_ord),order_disc(ord));       
          printf("\nIndex [oK:R]:   " );
	  integer_write(index);
          printf("\n");      
          integer_delref(index);

          printf ("(n  = %d, r1 = %d)\n",n, order_r1(sub_ord));
          printf("Order discriminant = ");
	  integer_write(order_disc(sub_ord));
	  printf("\n");

          printf ("TASK : \n");
          if (find_all) 
            printf ("find ALL solutions");                     
          else
            printf ("find ONE solution");                     

          printf    ("\nrel. T2 bounds:  ");
	  for (j=1; j<=sub_r1r2; j++)   
            real_write_aj (R, vec_entry(t2_bounds,j),10);

          printf    ("\nabs.. T2 bound:  ");
          real_write_aj (R, t2_bnd,10);

	  printf("\nlambda's:        ");
	  for (j=1; j<= sub_r1r2; j++)
           real_write_aj(R, vec_entry(lambdas,j), 10);


	  printf("\ngamma's:        ");
	  for (j=1; j<= sub_r1r2; j++)
           real_write_aj(R, vec_entry(*G,j), 10);

	  printf("\nrbounds for the ceilings: \n\n");
	  for (j=1; j<= sub_r1r2; j++)
	   { 
	     for (i=1; i<= m; i++)
              {
               printf("  (%d,", vec_entry(vec_entry(lbounds,j),i) );
               printf("%d)   ", vec_entry(vec_entry(ubounds,j),i) );  
              }

	     if (j<= sub_r1)
	      {
               sj= anf_rel_sort_real_zeroes_elt(sort,j);
               tj= anf_rel_sort_comp_zeroes_elt(sort,j);
	       printf("    mit   sj= %d, tj= %d\n", sj, tj);
	      }
	     else
	       printf("    mit   sj= -1, tj= -1\n");
            }
          printf("\n\n");
         }
               


/*** Loop with all possible r's ************************************************/
                                              
	/* Initialisation of the r vectors */       

	for (i=1; i<=sub_r1; i++)
	 {                 
	  vec_entry(all_ceils,i)= vec_new(m);
	  modi= 1;                                    
          sj= anf_rel_sort_real_zeroes_elt(sort,i);
          tj= anf_rel_sort_comp_zeroes_elt(sort,i);
          test= rel_anf_ceiling_fincke_next_sub(vec_entry(ubounds,i), 
                                              vec_entry(lbounds,i), 
					      &vec_entry(all_ceils,i), 
			                      sj, tj, &modi); 
	  if (!test)  break;
	 }

	for (i=sub_r1+1; i<=sub_r1r2; i++)
	 {                 
	  vec_entry(all_ceils,i)= vec_new(m);
	  modi= 1;                                    
          sj= -1;
          tj= -1;
          test= rel_anf_ceiling_fincke_next_sub(vec_entry(ubounds,i), 
                                              vec_entry(lbounds,i), 
					      &vec_entry(all_ceils,i), 
			                      sj, tj, &modi); 
	  if (!test)  break;
	 }
                                            


	/* Loop with all  r vectors */       

	modi= 0;
	sol_found= FALSE;
        while( test ) 
         {  
	  ceil_cnt++;

  	  /* Output of r */       

	  if (anf_print_level>=2)
	   {
            printf("\n\nceilings:  ");
	    for (j=1; j<=sub_r1r2; j++)
	     {    
              printf("(");
	      for (i=1; i<=m; i++)       
               {
	        printf(" %d  ", vec_entry(vec_entry(all_ceils,j),i));
               }
  	      printf(")");
  	      if (j<sub_r1r2)  printf(",   ");
	     }
	    printf("\n \n");     
	   }


  	  /* Determination of the quadratic forms */       
        
	  rel_abs_basis_weight= mat_new(nm,nm);

	  for (j=1; j<=sub_r1r2; j++)
          {     
           sj= anf_rel_sort_real_zeroes_elt(sort,j);
           tj= anf_rel_sort_comp_zeroes_elt(sort,j);
           if (j<= sub_r1) 
             ih1= sj+tj;
            else
             ih1= m;  
	   for (i=1; i<=ih1; i++)
	    {
             ind= anf_rel_sort_ordering_elt(sort,j,i);
             if (ind>r1+r2)
               ind= ind-r2;

             if ( ind <= r1)
               {
  	        temp1= real_power(R, vec_entry(lambdas,j), 
	                          vec_entry(vec_entry(all_ceils,j),i) );
                temp4 = real_sqrt (R, temp1);
     	        for (l=1; l<=nm; l++) 
                  mat_elt(rel_abs_basis_weight,ind,l)= 
                             real_mult(R,temp4, mat_elt(order_basis_real(rel_ord),ind,l) );

	        real_delete(&temp1);
	        real_delete(&temp4);
               }
              else
               {
                temp1= real_power(R, vec_entry(lambdas,j), 
	                        vec_entry(vec_entry(all_ceils,j),i) );
                if (j<= sub_r1)
                  temp2= real_power(R, vec_entry(lambdas,j), 
	                        vec_entry(vec_entry(all_ceils,j),i+tj) );
                 else
                  temp2= real_power(R, vec_entry(lambdas,j), 
	                        vec_entry(vec_entry(all_ceils,j),i) );


                temp3 = real_add (R, temp1, temp2);
                real_delete (&temp1);
                real_delete (&temp2);

                temp1= real_divide(R, temp3, two);
                temp4 = real_sqrt (R,temp1);
               
     	        for (l=1; l<=nm; l++) 
	         {                  
                  mat_elt(rel_abs_basis_weight,ind,l)= 
                          real_mult(R,temp4, mat_elt(order_basis_real(rel_ord),ind,l) );
                  mat_elt(rel_abs_basis_weight,ind+r2,l)= 
                          real_mult(R,temp4, mat_elt(order_basis_real(rel_ord),ind+r2,l) );
       	         }

                real_delete (&temp1);
                real_delete (&temp3);
                real_delete (&temp4);

               } /*  if (ind<= r1)  */

	    } /* for (i=1; i<=ih1; i++) */

	  }  /* for (j=1; j<=sub_r1r2; j++) */

          
	  if (anf_print_level >=3 )
	   {
            printf("\n\n\nOutput of alsolute weighted basis:\n\n");
	    mat_real_write_aj(R, rel_abs_basis_weight, 15);
	    printf("\n\n");     
	   }
 
           
 	  /* LLL reduction of the quadratic forms */       

          lll_trans= 0;
          lll_result= 0;
          lll_real_d_reduce(R, rel_abs_basis_weight,0.75, &lll_result, &lll_trans);

          lll_trans_int= mat_real_to_mat_z(R, lll_trans);
             
	  if (anf_print_level >=3 )
	   {
            printf("\n\n\nLLL-transformation:\n\n");
	    mat_real_write_aj(R, lll_trans, 15);
	    printf("\n\n");     

            printf("\n\n\nOutput of alsolute lll-red. weighted basis:\n\n");
	    mat_real_write_aj(R, lll_result, 15);
	    printf("\n\n");     
	   }
                   
          
          lat= lat_create();      
          lat_rank(lat)= nm;
          lat_basis_ring(lat)= ring_incref(R);
          lat_basis(lat)= lll_result;
          lat_chol_calc(lat);
          env= lat_enum_create(lat);
          lat_enum_ubound(env)= conv_int_to_real(R, 40); /*real_incref(t2_bnd);*/
          lat_enum_status_set_new   (env);
          lat_enum_request_set_next (env);
          lat_enum_strategy_set_up  (env);          


	  if (anf_print_level >=3 )
	   {
            printf("\nCholesky-matrix:\n");
	    mat_real_write_aj(R, lat_chol(lat), 15); 
	    printf("\n\n\n");
	   }


	  if (anf_print_level > 2)
	   {
	    printf("\n\nEnumeration loop:\n\n");
	    printf("\n\nUpper bound for the conjugated T2-norm:  \n\n");
	    real_write_aj(R, lat_enum_ubound(env) , 15);
	    printf("\n \n");
	   }


  	  /* Enumeration loop */  

	  while(lat_enum(lat, env))
	   {                             
	    lat_vec_cnt++;

            if (lat_vec_cnt==5000)   exit(1);

            lat_vec  = lat_elt_move (lat,lat_enum_act_coefs(env),lll_trans_int);
            rel_elt  = lat_elt_to_anf_elt (lat, lat_vec, rel_ord);
            lat_elt_delete(lat,&lat_vec);

	    if (anf_print_level >=3)
	     {  
	      printf("\nlat_enum_counter= \t %d\n", lat_enum_counter(env));
     	      printf(      "rel_elt with LLL-re-transformation:   \t ");
     	      anf_elt_write(rel_ord, rel_elt);

     	      printf("\nlength: ");  
              real_write_aj(R,lat_enum_act_length(env), 15);
              printf("\n \n");  
             }

	    norms= anf_elt_rel_norms(rel_ord, sub_ord, rel_elt, sort, abs_value); 

            if (anf_print_level>=0)
             {
	      /*  constructing the norm element in conjugated form */
 
              anf_con_alloc(alpha, n);
 
	      for (j=1; j<= sub_r1; j++)
	        anf_con(alpha, j) = comp_re(C, vec_entry(norms,j));

	      for (j=sub_r1+1; j<= sub_r1+sub_r2; j++)
               {                                  
                temp1= comp_re(C, vec_entry(norms,j));
                temp2= comp_im(C, vec_entry(norms,j));
	        anf_con(alpha, j) = real_mult(R, temp1, sq2);
	        anf_con(alpha, j+sub_r2) = real_mult(R, temp2, sq2);
                real_delete(&temp1);
                real_delete(&temp2);
               }

              printf("\nRelative norm element:  \n");
              anf_elt_embed_aj(sub_ord, alpha, 0, sub_ord, &elt_h3);

              anf_elt_write(sub_ord, elt_h3);
              printf("\n \n");
     
	      anf_elt_delete(sub_ord, &elt_h3);
	      anf_elt_delete(sub_ord, &alpha);
             }
            


	    if (FALSE)
              {
 	       if (anf_print_level >=3)
	        {
	         printf("norm:   ");
	         temp1= conv_int_to_real(R, 1);
     	         for (l=1;l<=sub_r1;l++) 
     	          {                                                            
	           temp2= temp1;
		   temp1= real_mult(R,temp2,vec_entry(norms,l));
	           real_delete(&temp2);
     	 	   real_write_aj(R, vec_entry(norms,l), 15);
		   if (l!=sub_r1r2)  printf(" * ");
     	          }                         
	         for (l=sub_r1+1;l<=sub_r1r2;l++) 
     	          {                                                            
	           temp2= temp1;  
	           temp3= real_power(R, vec_entry(norms,l), 2);
		   temp1= real_mult(R,temp2,temp3);
	           real_delete(&temp2);
	           real_delete(&temp3);
     		   real_write_aj(R, vec_entry(norms,l), 15);
		   printf("^2");
		   if (l!=sub_r1r2)  printf("  * ");
     	          }                          

     	         printf(" =  "); 
      	         real_write_aj(R, temp1, 15);
	         elt_h1= anf_elt_con(rel_ord, rel_elt);
	         anf_norm(rel_ord, elt_h1, &abs_norm, &abs_den);
	         anf_elt_delete(rel_ord, &elt_h1);
	         printf("   (absolute= ");
	         integer_write(abs_norm);
	         printf(")\n\n");
                 integer_delref(abs_norm);
                 integer_delref(abs_den);
	         real_delete(&temp1);   
	        }
                
	         erg= TRUE;
     	         for (j=1;j<=sub_r1r2;j++) 
     	          if (real_equality_eps(R,vec_entry(norms,j),vec_entry(K,j),loceps)==0)
	           {
	            erg= FALSE;
	            break;
	           }
             
	         vec_delete(R, &norms);
              }
	     else
              {
 	       if (anf_print_level >=0)
	        {
	         printf("norm:   ");
	         ctemp1= conv_int_to_comp(C, 1);
     	         for (l=1;l<=sub_r1;l++) 
     	          {                                                            
	           ctemp2= ctemp1;
		   ctemp1= comp_mult(C,ctemp2,vec_entry(norms,l));
	           comp_elt_delete(C, &ctemp2);
     	 	   comp_write_aj(C, vec_entry(norms,l), 15);
		   if (l!=sub_r1r2)  printf("\n * ");
     	          }                         
	         for (l=sub_r1+1;l<=sub_r1r2;l++) 
     	          {                                                            
	           ctemp2= ctemp1;  
	           ctemp3= comp_incref(vec_entry(norms,l));
		   ctemp4= comp_conjugate(C, ctemp3);
		   printf("(");
     		   comp_write_aj(C, ctemp3, 15);
	           printf(" * ");
     		   comp_write_aj(C, ctemp4, 15);
		   printf(")");
		   ctemp5= comp_mult(C, ctemp3, ctemp4);
		   ctemp1= comp_mult(C,ctemp2,ctemp5);
	           comp_elt_delete(C, &ctemp2);
	           comp_elt_delete(C, &ctemp3);
	           comp_elt_delete(C, &ctemp4);
	           comp_elt_delete(C, &ctemp5);
		   if (l!=sub_r1r2)  printf("\n       * ");
     	          }                          

     	         printf(" =  "); 
      	         comp_write_aj(C, ctemp1, 15);
	         elt_h1= anf_elt_con(rel_ord, rel_elt);
	         anf_norm(rel_ord, elt_h1, &abs_norm, &abs_den);
	         printf("   (absolute= ");
	         integer_write(abs_norm);
	         printf(")\n\n");
/*
                 printf("\nrel_elt, elt_h1 \n");
                 anf_elt_write(rel_ord, rel_elt);
                 printf("\n \n");
                 anf_elt_write(rel_ord, elt_h1);
                 printf("\n \n");
*/
                 integer_delref(abs_norm);
                 integer_delref(abs_den);
	         comp_elt_delete(C, &ctemp1);   
	         anf_elt_delete(rel_ord, &elt_h1);
	        }
                
	         erg= TRUE;
     	         if (comp_equality_eps(C, R, vec_entry(norms,1), norm_elt_c,loceps)==0)
	           erg= FALSE;
	         vec_delete(C, &norms);
	      }
    
	    if (erg)
	     {                    
              right_norm_cnt++;
	      unit_tst= FALSE;
              for (i=1; i<=dyn_arr_curr_length(*sol_list); i++)
	       {              
                elt_h1= anf_div(rel_ord, dyn_arr_element(*sol_list,i-1), rel_elt);
	        unit_tst= anf_elt_is_unit(rel_ord, elt_h1);
	        anf_elt_delete(rel_ord, &elt_h1);
	        if (unit_tst) break;
	       }
             
	      if (!unit_tst)
	       {
                dyn_arr_assure_space(*sol_list, dyn_arr_curr_length(*sol_list)+1,1);
                dyn_arr_curr_length(*sol_list)++; 
	        dyn_arr_element(*sol_list,dyn_arr_curr_length(*sol_list)-1)= 
	                                                          anf_elt_incref(rel_elt);


	        sol_found= TRUE;
	        if (find_all==0 )
	         {
	          anf_elt_delete(rel_ord, &rel_elt);
   	          mat_delref(Z, &lll_trans_int);
   	          mat_delref(R, &lll_trans);
   	          mat_delref(R, &rel_abs_basis_weight);

	          lat_enum_delete(lat, &env);
	          lat_delete(&lat);
	          goto end;
	         }
               }
	     } 

	    anf_elt_delete(rel_ord, &rel_elt);

	   }                     

   	  mat_delref(Z, &lll_trans_int);
   	  mat_delref(R, &lll_trans);
   	  mat_delref(R, &rel_abs_basis_weight);

	  lat_enum_delete(lat, &env);
	  lat_delete(&lat);


  	  /* New  r */       

	  for (i=1; i<=sub_r1r2; i++)
	   {                   
 	    if (i<=sub_r1)
	     {
              sj= anf_rel_sort_real_zeroes_elt(sort,i);
              tj= anf_rel_sort_comp_zeroes_elt(sort,i);
	     }
	    else
	     {
	      sj= -1;
	      tj= -1;
	     }
            test= rel_anf_ceiling_fincke_next_sub(vec_entry(ubounds,i), 
                                                  vec_entry(lbounds,i), 
	  			 	          &vec_entry(all_ceils,i), 
			                          sj, tj, &modi); 
	    if (test)  break;
	   }            
	   
	  if (!test) break;  /* no new r found, END */

	 } /* end while: test */




/*** End   *********************************************************************/


end:   for (j=1; j<=sub_r1r2; j++)
	 if (vec_entry(all_ceils,j)!=MEM_NH)
	   vec_delete(Z, &vec_entry(all_ceils,j) );
       vec_delete(Z, &all_ceils );

       for (j=1; j<=sub_r1r2; j++)
	 vec_delete(Z, &vec_entry(lbounds,j) );
       vec_delete(Z, &lbounds );

       for (j=1; j<=sub_r1r2; j++)
	 vec_delete(Z, &vec_entry(ubounds,j) );
       vec_delete(Z, &ubounds );                 
                      
       vec_delete(R, &lambdas );
       vec_delete(R, &t2_bounds);

       real_delete(&real_m);
       real_delete(&loceps);
       real_delete(&zero);
       real_delete(&two);
       real_delete(&sq2);
       real_delete(&t2_bnd);

       comp_elt_delete(C, &norm_elt_c);

       ring_delete(&Z);
       ring_delete(&C);  

       if (anf_print_level >=0)
        {
         puts ("Statistics :");
         printf (" No of ellip. considered             : %d\n", ceil_cnt);
         printf (" No of numbers found in these ellip. : %d\n", lat_vec_cnt);
         printf (" No of numbers found with right norm : %d\n", right_norm_cnt);
         if ((find_all) && (*sol_list != MEM_NH))
         printf (" No of non associated numbers        : %d\n",dyn_arr_curr_length (*sol_list));
         printf("\n \n");
        }


       if (sol_found)  return 1;
	
       return 0;
}


main()  
/*******************************************************************************
 
AJ Januar 1992
Last modification: -.

 
 
 
Description:
 
	Relative norm equation.
	
        Input format

         1)  Field 1 
         2)  Subfield
         3)  Which routines are called (0= both, 1= rel.., 2= abs....)
         4)  |N()| or N()  (0,1)
         4)  One solution or all solutions  (0,1)
         5)  Algebraic integer in the subfield for the relativ norm
         6)  If gammas are read or not (1,0)
         7)  r_1+r_2  Gamma values if in 5) 1 is read
        
        
	 
	Planned: -.

 
Calling sequence:
 
	-
 
History:
 
	92-06-2 AJ    written
 
*******************************************************************************/
{
	block_declarations;         
         
  	int			i,j, n, l, k, nm, m, int1, sj, tj; 
	int			sub_r1, sub_r2, sub_r1r2, subr, read_G, preci;
        double                  doub;
	t_logical		ok, abs_value, find_all;

	char			str[200];

   	t_handle          	R, Z, C;  
	order			ord, sub_ord, rel_ord, ord_h2;   
        anf_elt         	a_elt, a_elt_con, beta_elt, beta_elt_con; 

	t_real			t2_norm, h1, h2, h3, zero, loceps;  
	vector			rel_basis, rel_con;

	anf_rel_sort		sort;

	matrix			mat_h1;

	rel_lattice     	rel_lat;               
        rel_lat_enum_env	rel_env;            
	t_comp			comp_h1, comp, ctemp1, ctemp2, ctemp3, ctemp4, ctemp5;
	int 			lat_flag;

	lattice     	lat;               
        lat_enum_env	env;    

        /*  definitions for the moving of elements  */

	anf_elt			elt_h1, elt_h2, elt_h3, rel_elt, ord_elt, elt_h4, norm_elt;
	matrix			trans_pow_ord, trans_pow_sub_ord;
	matrix			trans_sub_ord_ord, trans_rel_ord;
	matrix			trans_ord_rel;
	matrix			mat_h2, mat_h3, mat_h4, mat_h5;
	integer_big 		det, trans_ord_rel_den, denom, abs_norm, abs_den;
	integer_big 		det_rel, prod_k;
	integer_small		erg, lauf1, lauf2, lauf3, lauf4, flag;
	vector			norms;
	dyn_arr_handle		store1, store2;

        /*  definitions for norm equations  */

	vector			K, G, dum1, dum2;
        dyn_arr_handle	        sol_list, normal_list;

/*** Initialisation  **********************************************************/
                                        
        preci= 30;

	kant_start();
	printf("\nfin.x ...... \n\n");

/*** Input loop  **************************************************************/
 
	while ( order_read(&ord) && order_read(&sub_ord) ) 
	{   
          /*** Initialisation of the orders  ***/
             
		order_reals_set(ord, preci);
		order_reals_create(ord);
		order_mult_table_create(ord); 
		nm=  order_abs_degree(ord);  

                R = order_reals(ord);            
		zero= conv_int_to_real(R,0); 
                loceps= real_make(R, 10, -real_dec_prec(R)+2 );  
		C = comp_create(real_dec_prec(R));
	        Z= m_z_str_incref(structure_z);

		order_reals_set(sub_ord, preci);
		order_reals_create(sub_ord);      

		ord_h2= order_lll_reduce(sub_ord);
                order_delete(&sub_ord); 
	        sub_ord= order_incref(ord_h2);
                order_delete(&ord_h2);        

		order_reals_set(sub_ord, preci);
		order_reals_create(sub_ord);
		order_mult_table_create(sub_ord);
		n=  order_abs_degree(sub_ord);
		sub_r1=  order_r1(sub_ord);
		sub_r2=  order_r2(sub_ord);
		sub_r1r2=  sub_r1+sub_r2;

		if (nm%n) 
		 {
		  printf("\n\nThe second field is not a subfield!\n\n");
		  printf(  "Terminated.\n\n");  

		  goto input_false;
		 }      
                                      
          /*** Construct a relative  basis  ***/

		m= nm/n;
		rel_basis= vec_new(m);  

		elt_h1= order_primitive_elt(ord);
	        for (i=1; i<=m; i++)      
	          vec_entry(rel_basis,i)= anf_power(ord, elt_h1, i-1);
	        anf_elt_delete(ord, &elt_h1);

                if (anf_print_level>=2)
                 {
                  printf("\nRelative Basis in reference to the integral basis:\n");
	          for (i=1; i<=m; i++)      
                   {
	             anf_elt_write(ord, vec_entry(rel_basis,i) );
                     printf("\n");
                   }
                 }
	

	


          /*** Shortest primitive element   ***/

		beta_elt= order_shortest_primitive_elt(sub_ord);  

                /*** Output of beta_elt           ***/

                beta_elt_con= anf_elt_con(sub_ord, beta_elt);  
               
		if (anf_print_level >=2 )
		 {
		  printf("The conjugates of a shortest primitive");
	  	  printf(" element in the sub_order:\n\n");
                  anf_elt_write(sub_ord,beta_elt); printf("  =  \n");
                  anf_elt_write(sub_ord,beta_elt_con); printf("\n\n");
		 }

                /*** Output of the T_2-norm of beta_elt              ***/

		t2_norm= anf_elt_t2(sub_ord, beta_elt);

		if (anf_print_level >=2 )
		 {
                  printf("T_2-norm of this element:                 "); 		 
                  real_write(R, t2_norm, 20); printf("\n\n\n\n\n\n");
		 }


          /*** Find the representation of beta_elt_con in the integral  ***/
          /*** basis of ord                                             ***/


		if (anf_elt_embed_aj(sub_ord, beta_elt, 0, ord, &a_elt))
 		 {
		  if (anf_print_level >=2 )
		   {
		    printf("Representation of this element in the extention order:\n\n");
		    printf("   ");
                    anf_elt_write(ord,a_elt); printf(" =  ");
                    elt_h1= anf_elt_con(ord, a_elt);
                    anf_elt_write(ord,elt_h1); printf("\n");
		    printf("\nT2-Norm of this element: ");
		    h1= anf_elt_t2(ord, a_elt);
                    real_write(R, h1, 20); 
		    printf("\n\n");
		    real_delete(&h1);                    
		    anf_elt_delete(ord, &elt_h1);
		   }
	   	 }
		else
		 {
		  printf("The second field is not a subfield!\n\n"); 
		  goto  no_sub_order;
		 }





          /*** Find the representation of the powers of the primitiv     ***/
	  /*** element in  sub_ord                                       ***/
                   
         	trans_pow_sub_ord= mat_new(n,n);
	
 		elt_h1= integer_anf_elt_lift(sub_ord, 1);
                anf_elt_to_mat_order_col(sub_ord, elt_h1, trans_pow_sub_ord, 1);
                anf_elt_to_mat_order_col(sub_ord, beta_elt, trans_pow_sub_ord, 2);
		anf_elt_delete(sub_ord, &elt_h1);

        	elt_h1= anf_elt_incref(beta_elt);
	        for (i=3; i<=n; i++)
		 {                  
		  elt_h2= elt_h1;
		  elt_h1= anf_mult(sub_ord, elt_h1, beta_elt);
		  anf_elt_delete(sub_ord, &elt_h2);
                  anf_elt_to_mat_order_col(sub_ord,elt_h1,trans_pow_sub_ord,i);
		 }
		anf_elt_delete(sub_ord, &elt_h1);  

		if (anf_print_level >=3 )
		 {
		  printf("\n\n\n\ntrans_pow_sub_ord= \n");
                  for (i=1; i<=n; i++)
		   {
                    for (j=1; j<=n; j++)
		      printf(" \t %d", mat_elt(trans_pow_sub_ord, i,j) );
	            printf("\n");
	           }
		  printf("\n \n");
		 }



          /*** Find the representation of the powers of the primitiv     ***/
	  /*** element of sub_ord in ord                                 ***/
                   
         	trans_pow_ord= mat_new(nm,n);
	
 		elt_h1= integer_anf_elt_lift(ord, 1);
                anf_elt_to_mat_order_col(ord, elt_h1, trans_pow_ord, 1);
                anf_elt_to_mat_order_col(ord, a_elt, trans_pow_ord, 2);
		anf_elt_delete(ord, &elt_h1);

        	elt_h1= anf_elt_incref(a_elt);
                for (i=3; i<=n; i++)
		 {            
		  elt_h2= elt_h1;
		  elt_h1= anf_mult(ord, elt_h1, a_elt);
		  anf_elt_delete(ord, &elt_h2);
                  anf_elt_to_mat_order_col(ord, elt_h1, trans_pow_ord, i);
		 }
		anf_elt_delete(ord, &elt_h1);


		if (anf_print_level >=3 )
		 {
		  printf("\n\ntrans_pow_ord= \n");
                  for (i=1; i<=nm; i++)
		   {
                    for (j=1; j<=n; j++)
		      printf(" \t %d", mat_elt(trans_pow_ord, i,j) );
	            printf("\n");
	           }
		  printf("\n \n");
		 }

          /*** Find the representation of the basis of sub_ord in ord    ***/

                mat_h2= mat_z_to_mat_real(R, trans_pow_ord);
                mat_h3= mat_z_to_mat_real(R, trans_pow_sub_ord);
                mat_h4= mat_ring_inverse(R,mat_h3);
		mat_h5= mat_ring_mult(R, mat_h2, mat_h4);  
	        trans_sub_ord_ord= mat_real_to_mat_z(R, mat_h5);

		if (anf_print_level >=3 )
		 {
		  printf("\n\nmat_h5= \n\n");
		  mat_real_write_aj(R, mat_h5, 15);
		  printf("\n \n");
		 }

		if (anf_print_level >=3 )
		 {
		  printf("\n\ntrans_sub_ord_ord= \n");
                  for (i=1; i<=nm; i++)
		   {
                    for (j=1; j<=n; j++)
		      printf(" \t %d", mat_elt(trans_sub_ord_ord, i,j) );
	            printf("\n");
	           }
		  printf("\n \n");
		 }

		mat_delref(R, &mat_h2);
		mat_delref(R, &mat_h3);
		mat_delref(R, &mat_h4);
		mat_delref(R, &mat_h5);
		                                    

          /*** Find the representation an relative element in ord       ***/
                                                                   
		trans_rel_ord= mat_new(nm, nm);
                for (i=1; i<=m; i++)
		 { 
                  elt_h1= anf_elt_incref(vec_entry(rel_basis,i));
                  for (j=1; j<=n; j++)
		   { 
		    elt_h3= mat_order_col_to_anf_elt(ord, trans_sub_ord_ord, j);
		    elt_h2= anf_mult(ord, elt_h1, elt_h3);
                    anf_elt_to_mat_order_col(ord, elt_h2, trans_rel_ord, (i-1)*n+j);
	  	    anf_elt_delete(ord, &elt_h2);
	  	    anf_elt_delete(ord, &elt_h3);
		   }
		  anf_elt_delete(ord, &elt_h1);
	         }

		if (anf_print_level >=3 )
		 {
		  printf("\n\ntrans_rel_ord= \n");
                  for (i=1; i<=nm; i++)
		   {
                    for (j=1; j<=nm; j++)
		      printf(" \t %d", mat_elt(trans_rel_ord, i,j) );
	            printf("\n");
	           }
		  printf("\n \n");
		 }
		 
		det= mat_z_det(Z, trans_rel_ord);
		det_rel= integer_incref(det);
		if (anf_print_level >=3 )
		 {
		  printf("\nDeterminant: ");
		  integer_write(det);
		  printf("\n \n");
		 }




          /*** Find the of ord in the relative basis if exists       ***/
                                                                   
		mat_h4= mat_z_to_mat_real(R, trans_rel_ord);
		h1=  conv_int_to_real(R, det); 
		h2= real_inverse(R, h1); 
		real_delete(&h1);
		h1= real_abs(R, h2);
		real_delete(&h2);
		mat_h5= mat_ring_scalar_right_mult(R,mat_h4,h1);
		mat_delref(R, &mat_h4);
		real_delete(&h1);
		mat_h4= mat_ring_inverse(R, mat_h5);

		if (anf_print_level >=3 )
        	 {
		  printf("\nmat_h4: \n\n");
		  mat_real_write_aj(R, mat_h4, 20);
		 }

	        trans_ord_rel= mat_real_to_mat_z(R, mat_h4);  
		trans_ord_rel_den= integer_abs(det);
		mat_delref(R, &mat_h4);
		mat_delref(R, &mat_h5);        


		mat_h4= mat_ring_mult(Z, trans_ord_rel, trans_rel_ord);
		if (anf_print_level >=3 )
		 {
		  printf("\n\nInverse test: \n");
                  for (i=1; i<=nm; i++)
		   {
                    for (j=1; j<=nm; j++)
		      printf(" \t %d", mat_elt(mat_h4, i,j) );
	            printf("\n");
	           }
		  printf("\n \n");
		 }
		mat_delref(Z, &mat_h4);


		if (anf_print_level >=3 )
		 {
		  printf("\n\ntrans_ord_rel= \n");
                  for (i=1; i<=nm; i++)
		   {
                    for (j=1; j<=nm; j++)
		      printf(" \t %d", mat_elt(trans_ord_rel, i,j) );
	            printf("\n");
	           }
		  printf("\n \n");
		 }

	        integer_delref(det);		 
		det= mat_z_det(Z, trans_rel_ord);
		if (anf_print_level >=3 )
		 {
		  printf("\nDenominator: ");
		  integer_write(trans_ord_rel_den);
		  printf("\n \n");
		 }
        
		
          /*** The order of relative basis multiplied by sub_ord basis  ***/

                if ( integer_abs(trans_ord_rel_den)==1 )
		  rel_ord= order_incref(ord);
                 else
		  rel_ord= order_trans_down(ord, trans_rel_ord);


	        integer_delref(det);		 
	        integer_delref(det_rel);		 
		
		 

          /*** Find the right order of the conjugates of ord over       ***/
          /***  sub_ord                                                 ***/
		 
                a_elt_con= anf_elt_con(ord, a_elt);  

		sort= anf_con_sort_sub(ord, sub_ord, a_elt_con, beta_elt_con); 

		if (anf_print_level >=2 )
		 {
                  printf("\n\n\n\nThe conjugates of this element in the suborder:\n");
                  anf_elt_write(sub_ord,beta_elt_con); printf("\n\n");
                  printf("\nThe conjugates of this element in the extention order:\n");
                  anf_elt_write(ord,a_elt_con); printf("\n\n");
  
                  printf("\nThe right order of the conjugates over the suborder:\n\n");
		  printf("   ");
		  for( j=1; j<= n; j++)   
	           {
		    for( i=1; i<= m; i++) 
		      printf("  %d ", anf_rel_sort_ordering_elt(sort,j,i));
	            printf("     ");
	           }
		  printf("\n");

	 	  printf("\n\n\nF^(j)'s without signature: %d \n", 2*order_r2(sub_ord));
	 	  printf("\n\n\nSignatures over F^(j): \n");
                  for (j= 1;j<= order_r1(sub_ord); j++) 
	  	   {
	 	    sj= anf_rel_sort_real_zeroes_elt(sort,j);
		    tj= anf_rel_sort_comp_zeroes_elt(sort,j);
		    printf("\nF^(%d):  s= %d,  t= %d ", j, sj, tj); 
		   }
                  printf("\n\n");

                  printf("\n\nThe sorted conjugates over the suborder:\n\n");  
		  for( j=1; j<= n; j++) 
	           {
		    for( i=1; i<=m; i++) 
		     {       
                      int1= anf_rel_sort_ordering_elt(sort,j,i);
		      comp_h1= anf_elt_ith_con(ord, a_elt_con, int1 );
		      comp_write_aj(C, comp_h1, 20);
		      printf("\n");
                      comp_elt_delete(C, &comp_h1);
		     }  
	 	    printf("\n");
	           }
		  printf("\n\n");                                             

		 }  /* endif: anf_print_level */

          /*** Computation of the conjugated relativ integral bases     ***/

		rel_con= rel_bases_con(ord, sub_ord, rel_basis, sort);
                
		if (anf_print_level >=2 )
		 {
                  printf("\n\n\nOutput of the conjugated relativ integral bases:\n\n");
		  for( i=1; i<= sub_r1r2; i++) 
		   {
	  	    printf("\n(i= %d):\n", i);
		    mat_comp_write_aj(C, vec_entry(rel_con,i), 20);
		    printf("\n\n");
		   }
		  printf("\n\n");     
		 }

          /*** Computation the solution of the norm equation     ***/


	
                printf ("(nm = %d, s = %d)\n",nm,order_r1(rel_ord));
                printf ("TASK : \n");


                /*** Which subroutine: both (0) or rel_order... (1) or  ***/
                /***                   order_lat... (2)                 ***/ 
                
		scanf("%d", &subr); 
		gets(str);

		if ( (subr<0) || (subr>2) )    
		  error_internal("spri.c: subr has a false value!");



		/* read abs_value */
               
	        scanf("%d", &abs_value);
		gets(str);
                if (abs_value==1)
		  printf("N^(j)( ) will be solve!\n");
                 else
		  printf("|N^(j)( )| will be solve!\n");
    

		/* read find_all */
               
	        scanf("%d", &find_all);
		gets(str);
                if (find_all==1)
		  printf("All solutions  of N^(j)( ) will be search!\n");
                 else
		  printf("One solutions  of N^(j)( ) will be search!\n");



		/* read the norms K in form of an algebraic integer of sub_ord */
                          
		K= vec_new(sub_r1r2);

		norm_elt= anf_elt_read(sub_ord);
		printf("relative norm: ");
	        anf_elt_write(sub_ord, norm_elt);                  
	        printf("\n"); 
	  
		printf("absolute conjugated relative norms:  ");
      	        for (i=1;i<=sub_r1r2;i++) 
 	         {
                  comp_h1= anf_elt_ith_con(sub_ord, norm_elt, i);
 	          h1= comp_norm(C, comp_h1);
	          vec_entry(K,i)= real_sqrt(R, h1);
	          real_write_aj(R, vec_entry(K,i), 15);
                  comp_elt_delete(C, &comp_h1);
                  real_delete(&h1);
	         }          
		printf("\n");


	        h1= conv_int_to_real(R, 1);
     	        for (i=1;i<=sub_r1;i++) 
     	         {                                                            
	          h2= h1;
		  h1= real_mult(R,h2,vec_entry(K,i));
	          real_delete(&h2);
     	         }                         
     	        for (i=sub_r1+1;i<=sub_r1r2;i++) 
     	         {                                                            
	          h2= h1;
	          h3= real_power(R, vec_entry(K,i), 2);
		  h1= real_mult(R,h2, h3);
	          real_delete(&h2);
	          real_delete(&h3);
     	         }                         

		prod_k= conv_real_to_int_round(R, h1);
		printf("absolute norm: %d \n", prod_k);

	        real_delete(&h1);



		/* read the Gammas G */
              
		scanf("%d", &read_G); 
		gets(str);

                if(read_G)
		  {
                   G= vec_new(sub_r1r2);
	   	   printf("gamma's: ");
                   for (i=1; i<= sub_r1r2; i++)
		    {
	             scanf("%lf", &doub);
		     gets(str); 
		     vec_entry(G,i)= conv_int_to_real(R, doub);
	             printf("  %lf  ", doub);
		    }                   
	           printf("\n");                
                  }  
                 else
                  G= MEM_NH;

		if (subr>1)  goto label_order_lat;


		printf("\nSolution of a relative norm equation:\n\n");

                ok= rel_order_lat_norm_equation_mod99(ord, sub_ord, rel_ord, rel_con, K, &G, sort,
        		                        &sol_list, find_all, norm_elt, abs_value);
                
		if (ok)
	         {
	          if (anf_print_level >= 0)
	           {    
		    printf("\nSolutions found: \n\n");
		    for (l=1; l<= dyn_arr_curr_length(sol_list); l++ )
		     {
     	              anf_elt_write(rel_ord, dyn_arr_element(sol_list,l-1) );
                      printf("\n");
                                         
                      rel_elt=  anf_elt_incref( dyn_arr_element(sol_list,l-1));

	              norms= anf_elt_rel_norms(rel_ord, sub_ord, rel_elt, sort, abs_value); 

	              if (abs_value==FALSE)
	                {
	 	         printf("norm:   ");
	                 h1= conv_int_to_real(R, 1);
     	                 for (j=1;j<=sub_r1;j++) 
     	                  {                                                            
	                   h2= h1;
		           h1= real_mult(R,h2,vec_entry(norms,j));
	                   real_delete(&h2);
     	 	           real_write_aj(R, vec_entry(norms,j), 15);
		           if (j!=sub_r1r2)  printf(" * ");
     	                  }                         
	                 for (j=sub_r1+1;j<=sub_r1r2;j++) 
     	                  {                                                            
	                   h2= h1;  
	                   h3= real_power(R, vec_entry(norms,j), 2);
		           h1= real_mult(R,h2,h3);
	                   real_delete(&h2);
	                   real_delete(&h3);
     		           real_write_aj(R, vec_entry(norms,j), 15);
		           printf("^2");
		           if (j!=sub_r1r2)  printf("  * ");
     	                  }                          

     	                 printf(" =  "); 
      	                 real_write_aj(R, h1, 15);
	                 elt_h1= anf_elt_con(rel_ord, rel_elt);
	                 anf_norm(rel_ord, elt_h1, &abs_norm, &abs_den);
	                 anf_elt_delete(rel_ord, &elt_h1);
	                 printf("   (absolute= ");
	                 integer_write(abs_norm);
	                 printf(")\n\n");
                         integer_delref(abs_norm);
                         integer_delref(abs_den);
	                 real_delete(&h1);   
             
        	         vec_delete(R, &norms);
                        }
	               else
                        {
	                 printf("norm:   ");
	                 ctemp1= conv_int_to_comp(C, 1);
     	                 for (j=1;j<=sub_r1;j++) 
     	                  {                                                            
	                   ctemp2= ctemp1;
	        	   ctemp1= comp_mult(C,ctemp2,vec_entry(norms,j));
	                   comp_elt_delete(C, &ctemp2);
     	         	   comp_write_aj(C, vec_entry(norms,j), 15);
	        	   if (j!=sub_r1r2)  printf("\n * ");
     	                  }                         
	                 for (j=sub_r1+1;j<=sub_r1r2;j++) 
     	                  {                                                            
	                   ctemp2= ctemp1;  
	                   ctemp3= comp_incref(vec_entry(norms,j));
	        	   ctemp4= comp_conjugate(C, ctemp3);
	        	   printf("(");
     	        	   comp_write_aj(C, ctemp3, 15);
	                   printf(" * ");
     	        	   comp_write_aj(C, ctemp4, 15);
	        	   printf(")");
	        	   ctemp5= comp_mult(C, ctemp3, ctemp4);
	        	   ctemp1= comp_mult(C,ctemp2,ctemp5);
	                   comp_elt_delete(C, &ctemp2);
	                   comp_elt_delete(C, &ctemp3);
	                   comp_elt_delete(C, &ctemp4);
	                   comp_elt_delete(C, &ctemp5);
	        	   if (j!=sub_r1r2)  printf("\n  * ");
     	                  }                          

     	                 printf(" =  "); 
      	                 comp_write_aj(C, ctemp1, 15);
	                 elt_h1= anf_elt_con(rel_ord, rel_elt);
	                 anf_norm(rel_ord, elt_h1, &abs_norm, &abs_den);
	                 anf_elt_delete(rel_ord, &elt_h1);
	                 printf("   (absolute= ");
	                 integer_write(abs_norm);
	                 printf(")\n\n");
                         integer_delref(abs_norm);
                         integer_delref(abs_den);
	                 comp_elt_delete(C, &ctemp1);   

	                 vec_delete(C, &norms);  
	                }
                       
		       anf_elt_delete(rel_ord, &rel_elt);

	             }                      
	           }
                 }
		else
		  printf("\nNo solution found! \n");


		for (i=1; i<=dyn_arr_curr_length(sol_list); i++)
		 {
		  anf_elt_delete(rel_ord, &dyn_arr_element(sol_list,i-1));
                
		  dyn_arr_element(sol_list,i-1)= 0;
	         }
		dyn_arr_delete(&sol_list);   


          /*** Normal norm equation  ***/ 
                                                   
		if (subr==1)  goto deleting_storage;

label_order_lat:

		printf("\nSolution of the norm equation by Fincke:\n\n");

/*
                  rel_order_fincke_find_gamma_opt(sub_ord, K, &dum1, &dum2, sort);           
                  vec_delete(R, &dum1);
                  vec_delete(R, &dum2);
*/
            
                elt_h1= MEM_NH;
                normal_list= MEM_NH;

	     	order_norm_equation_aj(rel_ord, sub_ord,prod_k, &elt_h1, 
                                       &normal_list, find_all,abs_value,sort, K, norm_elt);

		if ((normal_list != MEM_NH))
		 {
		  printf ("\nSolutions of normal norm equation : \n\n");
		  for (l=1;l<=dyn_arr_curr_length (normal_list);l++)  
		   {                     
		    anf_elt_write (rel_ord,dyn_arr_element (normal_list,l-1));		
		    printf("\n");                          
		    rel_elt= dyn_arr_element (normal_list,l-1);
	            norms= anf_elt_rel_norms(rel_ord, sub_ord, 
			     rel_elt, sort, abs_value); 
                      

 	            if (abs_value==FALSE)
	              {
	 	       printf("norm:   ");
	               h1= conv_int_to_real(R, 1);
     	               for (j=1;j<=sub_r1;j++) 
     	                {                                                            
	                 h2= h1;
		         h1= real_mult(R,h2,vec_entry(norms,j));
	                 real_delete(&h2);
     	 	         real_write_aj(R, vec_entry(norms,j), 15);
		         if (j!=sub_r1r2)  printf("     * ");
     	                }                         
	               for (j=sub_r1+1;j<=sub_r1r2;j++) 
     	                {                                                            
	                 h2= h1;  
	                 h3= real_power(R, vec_entry(norms,j), 2);
		         h1= real_mult(R,h2,h3);
	                 real_delete(&h2);
	                 real_delete(&h3);
     		         real_write_aj(R, vec_entry(norms,j), 15);
		         printf("^2");
		         if (j!=sub_r1r2)  printf("      * ");
     	                }                          

     	               printf(" =  "); 
      	               real_write_aj(R, h1, 15);
	               elt_h1= anf_elt_con(rel_ord, rel_elt);
	               anf_norm(rel_ord, elt_h1, &abs_norm, &abs_den);
	               anf_elt_delete(rel_ord, &elt_h1);
	               printf("   (absolute= ");
	               integer_write(abs_norm);
	               printf(")\n\n");
                       integer_delref(abs_norm);
                       integer_delref(abs_den);
	               real_delete(&h1);   
                       
        	       vec_delete(R, &norms);
                      }
	             else
                      {
	               printf("norm:   ");
	               ctemp1= conv_int_to_comp(C, 1);
     	               for (j=1;j<=sub_r1;j++) 
     	                {                                                            
	                 ctemp2= ctemp1;
	                 ctemp1= comp_mult(C,ctemp2,vec_entry(norms,j));
	                 comp_elt_delete(C, &ctemp2);
     	                 comp_write_aj(C, vec_entry(norms,j), 15);
	                 if (j!=sub_r1r2)  printf("\n * ");
     	                }                         
	               for (j=sub_r1+1;j<=sub_r1r2;j++) 
     	                {                                                            
	                 ctemp2= ctemp1;  
	                 ctemp3= comp_incref(vec_entry(norms,j));
	                 ctemp4= comp_conjugate(C, ctemp3);
	                 printf("(");
     	                 comp_write_aj(C, ctemp3, 15);
	                 printf(" * ");
     	                 comp_write_aj(C, ctemp4, 15);
	                 printf(")");
	                 ctemp5= comp_mult(C, ctemp3, ctemp4);
	                 ctemp1= comp_mult(C,ctemp2,ctemp5);
	                 comp_elt_delete(C, &ctemp2);
	                 comp_elt_delete(C, &ctemp3);
	                 comp_elt_delete(C, &ctemp4);
	                 comp_elt_delete(C, &ctemp5);
	                 if (j!=sub_r1r2)  printf("\n  * ");
     	                }                          

     	               printf(" =  "); 
      	               comp_write_aj(C, ctemp1, 15);
	               elt_h1= anf_elt_con(rel_ord, rel_elt);
	               anf_norm(rel_ord, elt_h1, &abs_norm, &abs_den);
	               anf_elt_delete(rel_ord, &elt_h1);
	               printf("   (absolute= ");
	               integer_write(abs_norm);
	               printf(")\n\n");
                       integer_delref(abs_norm);
                       integer_delref(abs_den);
	               comp_elt_delete(C, &ctemp1);   

	               vec_delete(C, &norms);
	              }

		   }

		  for (i=1; i<=dyn_arr_curr_length(normal_list); i++)
		   {
		    anf_elt_delete(rel_ord, &dyn_arr_element(normal_list,i-1) );
		    dyn_arr_element(normal_list,i-1)= 0;
	           }
		  dyn_arr_delete(&normal_list);   
		 }
		else
		  if (elt_h1==MEM_NH) 
                    puts ("NO SOLUTION");
                   else
                    {
                     printf("\nOne Solution: \n");
                     anf_elt_write(rel_ord, elt_h1);
                     printf("\n \n");
                    }
                  
	  	printf("\n \n");

	  	if (elt_h1!=MEM_NH)  anf_elt_delete(rel_ord, &elt_h1);
	
		integer_delref(prod_k);




          /*** End of while loop: delete storage  ***/     

deleting_storage:
	
	        anf_elt_delete(sub_ord, &norm_elt);

		anf_rel_sort_delete(&sort); 

		for (i=1; i<=sub_r1r2; i++)
		 {
		  real_delete(&vec_entry(K,i));
		  vec_entry(K,i)= 0;
	         }
		vec_delete(Z, &K);   


                if (subr<2)
		  vec_delete(R, &G);   


		for (i=1; i<=sub_r1r2; i++)
		 {
		  mat_delref(C, &vec_entry(rel_con,i));
		  vec_entry(rel_con,i)= 0;
		 }
		vec_delete(Z, &rel_con);      


		mat_delref(Z, &trans_pow_ord);
		mat_delref(Z, &trans_pow_sub_ord);
		mat_delref(Z, &trans_sub_ord_ord);
		mat_delref(Z, &trans_rel_ord);
		mat_delref(Z, &trans_ord_rel);
		integer_delref(trans_ord_rel_den);



		anf_elt_delete (ord,&a_elt);
		anf_elt_delete (ord,&a_elt_con);  

 
		real_delete(&t2_norm);  


 		order_delete(&rel_ord); 
                        
no_sub_order:	
		anf_elt_delete (sub_ord,&beta_elt);	 
		anf_elt_delete (sub_ord,&beta_elt_con); 

		for (i=1; i<=m; i++)
		 {
		  anf_elt_delete(ord, &vec_entry(rel_basis,i));
		  vec_entry(rel_basis,i)= 0;
	         }
		vec_delete(Z, &rel_basis);   

input_false:	real_delete(&zero);
		real_delete(&loceps);
		order_delete(&ord);
 		order_delete(&sub_ord); 

		ring_delete(&C);
		ring_delete(&Z);
                          
	} 

/*** End  *********************************************************************/
 

}         


